unit HID_Intf;

// Version 2013-01-19

{
  USB: Human Interface Device class interface file:
  Basic calls to communicate with an HID device

  When using this unit/library make sure the next "standard"
  Windows DLL's are installed on the target system:
     - HID.DLL
     - SetupAPI.DLL

  Created:     16-04-2012 by J.W. Beunders
  Last update: 17-08-2012 by J.W. Beunders (joop@beunders.be)

  This code may be used freely. The author would appreciate it
  when he is mentioned as original creator of the interface.
  Also he would be very grateful if he is notified when changes
  and / or improvements are made.

  Extended (11-12-2012) by D. Rosseel to be able to use
   - report ID's
   - SetOutputReport
   - GetInputReport
   - SetFeatureReport
   - GetFeatureReport
}

interface

type
{
  Basic HID data buffer. This the maximum size of an HID buffer.
}
  THIDbuffer = array[0..63] of Byte;

{
  Basic HID report: consists of a 64 byte Databuffer and a ReportID
}
  THIDreport =
    record
    ReportID: Byte; // the report ID
    Data: THIDbuffer; // the data buffer
  end;

{
  The interface includes a thread which tries to read data from
  the active HID device. When a callback procdure is assigned in
  the main program, all available data is automatically returned
  to the application in this procedure. The report can be found
  in "Data", the number of meaningfull bytes in "NumBytes".
}
  THIDcallBack = procedure(Data: THIDReport; NumBytes: byte); stdcall;

{
  When a USB device is attached or removed from the system, the
  system will generate a message. By specifying a callback procedure
  in the main program, the event can be used to take some action,
  for instance to check if the active device is still present.
}
  TUSBcallBack = procedure; stdcall;

{
  The report capabilities per ReportID (simplified) are defined below.
}
  TReportCapabilities =
    record
    Usage       : word; // usage
    ReportId    : byte; // Report Id
    ReportSize  : byte; // report size: # bits per report
    ReportCount : byte; // reportcount: nr of bytes in the report belonging to ReportId and Usage
  end;

const
  MaxNrCapabilities = 10;

{
  An HID device is specified (simplified) by the structure shown below. Apart from
  the VID/PID combination, the buffersizes are of great importance
  as that specifies how much data can be transferred in a single call.
}
{
  The structure below contains information of all HID devices after a call
  to ScanForHIDdevices.
}
type
  THIDdeviceInfo = record
    SymLink: ShortString; // symbolic link from registry
    OutputBufferSize: Word;
    InputBufferSize: Word;
    FeatureBufferSize: word;
    OutputReportCaps: array[0..MaxNrCapabilities - 1] of TReportCapabilities;
    InputReportCaps: array[0..MaxNrCapabilities - 1] of TReportCapabilities;
    FeatureReportCaps: array[0..MaxNrCapabilities - 1] of TReportCapabilities;
    Handle: THandle; // handle which is returned when device is opened
    VID: Word; // Vendor ID code
    PID: Word; // Product ID code
    VersionNumber: Word; // Version information
    ManufacturerString: ShortString; // Textual manufacurer information
    ProductString: ShortString; // Textual product information
    SerialNumberString: ShortString; // Textual serial number information
    HasReadWriteAccess: boolean;
  end;

{
  The maximum number of devices that are stored after "ScanForHIDdevices"
}
const
  MaxNrDevices = 32; // 32 devices maximum

{
  Assign procedure to call when a usb device is inserted or removed
}
procedure USBsetEventHandler(Callback: TUsbCallBack); stdcall;

{
  Assign a user procedure which will receive data when HID device has send it
}
procedure HIDsetEventHandler(CallBack: THidCallBack); stdcall;

{
  Scan all available HID Devices.
  All HID entries are stored in a device table.
  The value returned is the number of devices found. 
}
function HIDscanForDevices: byte; stdcall;

{
  Get the info for the device with Nr (0..MaxNrDevices -1)
}
procedure HIDGetDeviceInfo(Nr: byte; var data: THIDDeviceInfo); stdcall;

{
  Open a device (0..MaxNrDevices -1).
  Returns TRUE if successfull.
}
function HIDopenDevice(DeviceNr: byte): Boolean; stdcall;

{
  Close Device; close the open device.
  Returns TRUE if successfull.
}
function HIDcloseDevice: Boolean; stdcall;

{
  Write a HID report to HID device.
  Returns the number of meaningfull bytes sent to the device.
}
function HIDwriteDevice(Report: THIDReport): byte; stdcall;

{
  Read a HID report from device with a time-out.
  Returns the number of meaningfull bytes read from the device.
}
function HIDreadDevice(var OutReport: THIDReport): byte; stdcall;

{
  Get the size of the output report (PC --> device), always including an extra report ID byte,
  This size is defined by the largest output report size if more reportID's are used.
}
function HIDOutputReportSize: byte; stdcall;

{
  Get the size of the input report (PC --> device), always including an extra report ID byte,
  This size is defined by the largest input report size if more reportID's are used.
}
function HIDInputReportSize: byte; stdcall;

{
  Get the size of the Feature report (PC --> device), always including an extra report ID byte,
  This size is defined by the largest feature report size if more reportID's are used.
}
function HIDFeatureReportSize: byte; stdcall;

{
  Get the total Output report size related to the report with ReportId, always excluding the report ID byte.
  Gives the number of (meaningfull) data bytes (excl the reportID) actually transferred.
}
function HIDOutputReportCount(_ReportId: byte): byte; stdcall;

{
  Get the total Input report size related to the report with _ReportId, always excluding the report ID byte.
  Gives the number of (meaningfull) data bytes (excl the reportID) actually transferred.
}
function HIDInputReportCount(_ReportId: byte): byte; stdcall;

{
  Get the total Feature report size related to the report with _ReportId, always excluding the report ID byte.
  Gives the number of (meaningfull) data bytes (excl the reportID) actually transferred.
}
function HIDFeatureReportCount(_ReportId: byte): byte; stdcall;

{
  Sends the output report "OutReport" to the USB device.
  Returns the number of meaningfull databytes written to the device.
}
function HIDSetOutputReport(var OutReport: THIDReport): byte; stdcall;

{
  Gets the input report "InReport" from the USB device.
  Returns the number of meaningfull databytes read from the device.
}
function HIDGetInputReport(var InReport: THIDReport): byte; stdcall;

{
 Sends the feature report "OutReport" to the USB device.
 Returns the number of meaningfull databytes written to the device.
}
function HIDSetFeatureReport(var OutReport: THIDReport): byte; stdcall;

{
 Gets the feature report "InReport" to the USB device.
 Returns the number of meaningfull databytes read from the device.
}
function HIDGetFeatureReport(var InReport: THIDReport): byte; stdcall;


implementation

procedure USBsetEventHandler(Callback: TUsbCallBack);               stdcall; External 'HID_Library__.dll';
procedure HIDsetEventHandler(CallBack: THIDCallBack);               stdcall; External 'HID_Library__.dll';
function HIDscanForDevices:byte;                                    stdcall; External 'HID_Library__.dll';
procedure HIDGetDeviceInfo(Nr: byte; var data: THIDDeviceInfo);     stdcall; External 'HID_Library__.dll';
function HIDopenDevice(DeviceNr: byte): Boolean;                    stdcall; External 'HID_Library__.dll';
function HIDcloseDevice: Boolean;                                   stdcall; External 'HID_Library__.dll';
function HIDwriteDevice(Report: THIDReport): byte;                  stdcall; External 'HID_Library__.dll';
function HIDreadDevice(var OutReport: THIDReport): byte;            stdcall; External 'HID_Library__.dll';
function HIDOutputReportSize: byte;                                 stdcall; External 'HID_Library__.dll';
function HIDInputReportSize: byte;                                  stdcall; External 'HID_Library__.dll';
function HIDFeatureReportSize: byte;                                stdcall; External 'HID_Library__.dll';
function HIDOutputReportCount(_ReportId: byte): byte;               stdcall; External 'HID_Library__.dll';
function HIDInputReportCount(_ReportId: byte): byte;                stdcall; External 'HID_Library__.dll';
function HIDFeatureReportCount(_ReportId: byte): byte;              stdcall; External 'HID_Library__.dll';
function HIDSetOutputReport(var OutReport: THIDReport): byte;       stdcall; External 'HID_Library__.dll';
function HIDGetInputReport(var InReport: THIDReport): byte;         stdcall; External 'HID_Library__.dll';
function HIDSetFeatureReport(var OutReport: THIDReport): byte;      stdcall; External 'HID_Library__.dll';
function HIDGetFeatureReport(var InReport: THIDReport): byte;       stdcall; External 'HID_Library__.dll';

end.
